@@ -10,7 +10,7 @@ from os import remove, stat
 from os.path import isfile
 from time import sleep, time
 
-from .status import ReducerStats, Status
+from .status import Status
 
 
 def test_status_01(tmp_path):
@@ -130,7 +130,7 @@ def test_status_07(tmp_path):
     status.timestamp += 1
     assert status.rate == 0.5
 
-def _client_writer(done, working_path):
+def _client_writer(done, reported, working_path):
     """Used by test_status_08"""
     # NOTE: this must be at the top level to work on Windows
     Status.PATH = working_path
@@ -139,6 +139,9 @@ def _client_writer(done, working_path):
         while not done.is_set():
             status.iteration += 1
             status.report(force=True)
+            # perform two reports before setting flag
+            if not reported.is_set() and status.iteration > 1:
+                reported.set()
             sleep(0.01)
     finally:
         status.cleanup()
@@ -146,94 +149,26 @@ def _client_writer(done, working_path):
 def test_status_08(tmp_path):
     """test Status.loadall() with multiple active reporters"""
     Status.PATH = str(tmp_path)
-    best_rate = 0
     done = Event()
     procs = list()
+    report_events = list()
     try:
+        # launch processes
         for _ in range(5):
-            procs.append(Process(target=_client_writer, args=(done, Status.PATH)))
+            report_events.append(Event())
+            procs.append(Process(target=_client_writer, args=(done, report_events[-1], Status.PATH)))
             procs[-1].start()
-        deadline = time() + 60
-        while len(tuple(Status.loadall())) < len(procs):
-            sleep(0.1)
-            assert time() < deadline, "timeout waiting for processes to launch!"
-        for _ in range(20):
-            for obj in Status.loadall():
-                if obj.rate > best_rate:
-                    best_rate = obj.rate
+        # wait for processes to launch and report
+        for has_reported in report_events:
+            assert has_reported.wait(timeout=60)
+        # collect reports
+        reports = tuple(Status.loadall())
+        assert len(reports) == len(procs)
+        assert max(x.rate for x in reports) > 0
     finally:
         done.set()
         for proc in procs:
             if proc.pid is not None:
                 proc.join()
-    assert best_rate > 0
+    # verify cleanup
     assert not any(Status.loadall())
-
-def test_reducer_stats_01(tmp_path):
-    """test ReducerStats() empty"""
-    ReducerStats.PATH = str(tmp_path)
-    with ReducerStats() as stats:
-        assert stats.error == 0
-        assert stats.failed == 0
-        assert stats.passed == 0
-        stats_file = stats._file
-        assert not isfile(stats_file)
-    assert isfile(stats_file)
-
-def test_reducer_stats_02(tmp_path):
-    """test ReducerStats() simple"""
-    ReducerStats.PATH = str(tmp_path)
-    with ReducerStats() as stats:
-        stats.error += 1
-        stats.failed += 1
-        stats.passed += 1
-    with ReducerStats() as stats:
-        assert stats.error == 1
-        assert stats.failed == 1
-        assert stats.passed == 1
-
-def test_reducer_stats_03(tmp_path):
-    """test ReducerStats() empty/incomplete/invalid data file"""
-    ReducerStats.PATH = str(tmp_path)
-    stats_file = tmp_path / ReducerStats.FILE
-    # missing file
-    with ReducerStats() as stats:
-        stats.passed += 1
-    # invalid empty file
-    stats_file.write_bytes(b"")
-    with ReducerStats() as stats:
-        assert stats.passed == 0
-    # incomplete file
-    stats_file.write_bytes(b"{}")
-    with ReducerStats() as stats:
-        assert stats.passed == 0
-
-def _reducer_client(working_path, limit, unrestrict):
-    """Used by test_reducer_stats_04"""
-    # NOTE: this must be at the top level to work on Windows
-    ReducerStats.PATH = working_path
-    for _ in range(50):
-        with ReducerStats() as stats:
-            stats.passed += 1
-            if stats.passed == limit:
-                unrestrict.set()
-        unrestrict.wait(timeout=60)
-
-def test_reducer_stats_04(tmp_path):
-    """test ReducerStats() with multiple processes"""
-    ReducerStats.PATH = str(tmp_path)
-    procs = list()
-    unrestrict = Event()  # used to sync client procs
-    try:
-        proc_count = 5
-        for _ in range(proc_count):
-            procs.append(Process(
-                target=_reducer_client, args=(ReducerStats.PATH, proc_count, unrestrict)))
-            procs[-1].start()
-    finally:
-        unrestrict.set()
-        for proc in procs:
-            if proc.pid is not None:
-                proc.join()
-    with ReducerStats() as stats:
-        assert stats.passed == 250
