@@ -3,7 +3,7 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-__all__ = ("ReplayManager", "ReplayArgs")
+__all__ = ("ReplayManager", "ReplayArgs", "ReplayResult")
 
-from .replay import ReplayManager
+from .replay import ReplayManager, ReplayResult
 from .args import ReplayArgs
