@@ -10,7 +10,7 @@ from ..args import CommonArgs
 class ReplayArgs(CommonArgs):
 
     def __init__(self):
-        super(ReplayArgs, self).__init__()
+        super().__init__()
         self.parser.add_argument(
             "input",
             help="Accepted input includes: " \
@@ -48,6 +48,11 @@ class ReplayArgs(CommonArgs):
         replay_args.add_argument(
             "--sig",
             help="Signature (JSON) file to match.")
+        replay_args.add_argument(
+            "--test-index", type=int, nargs="+",
+            help="Select a testcase to run when multiple testcases are loaded. " \
+                 "Testscases are ordered oldest to newest. Indexing is 0 based. " \
+                 "0 == Oldest, n-1 == Newest (default: run all testcases)")
 
         self.launcher_grp.add_argument(
             "--rr", action="store_true",
@@ -58,7 +63,7 @@ class ReplayArgs(CommonArgs):
             help="Include the testcase when reporting results.")
 
     def sanity_check(self, args):
-        super(ReplayArgs, self).sanity_check(args)
+        super().sanity_check(args)
 
         if args.any_crash and args.sig is not None:
             self.parser.error("signature is ignored when running with '--any-crash'")
