@@ -6,7 +6,6 @@
 from logging import getLogger
 from os.path import dirname, join as pathjoin
 from tempfile import mkdtemp
-from time import sleep
 from shutil import rmtree
 
 from FTB.Signatures.CrashInfo import CrashSignature
@@ -18,6 +17,7 @@ from ..common.status import Status
 from ..common.storage import TestCase, TestCaseLoadFailure, TestFile
 from ..common.utils import grz_tmp
 from ..main import configure_logging
+from ..session import Session
 from ..target import load as load_target, TargetLaunchError, TargetLaunchTimeout
 
 __author__ = "Tyson Smith"
@@ -26,7 +26,7 @@ __credits__ = ["Tyson Smith"]
 LOG = getLogger(__name__)
 
 
-class ReplayResult(object):
+class ReplayResult:
     __slots__ = ("count", "durations", "expected", "report", "served")
 
     def __init__(self, report, served, durations, expected):
@@ -37,24 +37,28 @@ class ReplayResult(object):
         self.served = served
 
 
-class ReplayManager(object):
+class ReplayManager:
     HARNESS_FILE = pathjoin(dirname(__file__), "..", "common", "harness.html")
 
     __slots__ = ("ignore", "server", "status", "target", "_any_crash",
-                 "_harness", "_signature", "_unpacked")
+                 "_harness", "_signature", "_relaunch", "_unpacked")
 
-    def __init__(self, ignore, server, target, any_crash=False, signature=None, use_harness=True):
+    def __init__(self, ignore, server, target, any_crash=False,
+                 relaunch=1, signature=None, use_harness=True):
         self.ignore = ignore
         self.server = server
         self.status = None
         self.target = target
         self._any_crash = any_crash
         self._harness = None
-        # TODO: make signature a property
+        self._relaunch = relaunch
         self._signature = signature
         if use_harness:
             with open(self.HARNESS_FILE, "rb") as in_fp:
                 self._harness = in_fp.read()
+        else:
+            # target must relaunch every iteration when not using harness
+            assert relaunch == 1
 
     def __enter__(self):
         return self
@@ -62,6 +66,10 @@ class ReplayManager(object):
     def __exit__(self, *exc):
         self.cleanup()
 
+    @property
+    def signature(self):
+        return self._signature
+
     def cleanup(self):
         """Remove temporary files from disk.
 
@@ -74,6 +82,36 @@ class ReplayManager(object):
         if self.status is not None:
             self.status.cleanup()
 
+    @classmethod
+    def load_testcases(cls, path, load_prefs, subset=None):
+        """Load TestCases.
+
+        Args:
+            path (str): Path to load.
+            load_prefs (bool): Load prefs.js file if available.
+            subset (list(int)): Indices of tests to load when loading multiple
+                                tests.
+        Returns:
+            list(TestCase): Loaded TestCases.
+        """
+        LOG.debug("loading the TestCases")
+        testcases = TestCase.load(path, load_prefs)
+        if not testcases:
+            raise TestCaseLoadFailure("Failed to load TestCases")
+        if subset:
+            count = len(testcases)
+            # deduplicate and limit requested indices to valid range
+            reqs = {max(count + x, 0) if x < 0 else min(x, count - 1) for x in subset}
+            LOG.debug("using TestCase(s) with index %r", reqs)
+            selected = list()
+            for idx in sorted(reqs, reverse=True):
+                selected.append(testcases.pop(idx))
+            selected.reverse()
+            for test in testcases:
+                test.cleanup()
+            testcases = selected
+        return testcases
+
     @staticmethod
     def report_to_filesystem(path, results, tests=None):
         """Use FilesystemReporter to write reports and testcase to disk in a
@@ -104,7 +142,7 @@ class ReplayManager(object):
             for result in expected:
                 reporter.submit(tests or [], report=result.report)
 
-    def run(self, testcases, repeat=1, min_results=1, idle_delay=0, idle_threshold=0):
+    def run(self, testcases, repeat=1, min_results=1, exit_early=True, idle_delay=0, idle_threshold=0):
         """Run testcase replay.
 
         Args:
@@ -112,6 +150,11 @@ class ReplayManager(object):
             repeat (int): Maximum number of times to run the TestCase.
             min_results (int): Minimum number of results needed before run can
                                be considered successful.
+            exit_early (bool): If True the minimum required number of iterations
+                               are performed to either meet `min_results` or
+                               determine that it is not possible to do so.
+                               If False `repeat` number of iterations are
+                               performed.
             idle_delay (int): Number of seconds to wait before polling for idle.
             idle_threshold (int): CPU usage threshold to mark the process as idle.
 
@@ -124,6 +167,7 @@ class ReplayManager(object):
         assert repeat > 0
         assert repeat >= min_results
         assert testcases
+        assert len(testcases) == 1 or self._harness is not None
 
         if self.status is not None:
             LOG.debug("clearing previous status data")
@@ -132,16 +176,8 @@ class ReplayManager(object):
 
         server_map = ServerMap()
         if self._harness is not None:
-            def _dyn_close():  # pragma: no cover
-                if self.target.monitor.is_healthy():
-                    # delay to help catch window close/shutdown related crashes
-                    sleep(0.1)
-                    self.target.close()
-                return b"<h1>Close Browser</h1>"
-            server_map.set_dynamic_response("grz_close_browser", _dyn_close, mime_type="text/html")
             server_map.set_dynamic_response("grz_harness", lambda: self._harness, mime_type="text/html")
 
-        runner = Runner(self.server, self.target, idle_threshold=idle_threshold, idle_delay=idle_delay)
         # track unprocessed results
         reports = dict()
         # track unpacked testcases
@@ -154,11 +190,17 @@ class ReplayManager(object):
                 dst_path = mkdtemp(prefix="tc_", dir=grz_tmp("serve"))
                 unpacked.append(dst_path)
                 test.dump(dst_path)
+            relaunch = min(self._relaunch, repeat)
+            runner = Runner(
+                self.server,
+                self.target,
+                idle_threshold=idle_threshold,
+                idle_delay=idle_delay,
+                relaunch=relaunch * test_count)
             # perform iterations
             for _ in range(repeat):
                 self.status.iteration += 1
                 if self.target.closed:
-                    LOG.info("Launching target...")
                     if self._harness is None:
                         location = runner.location(
                             "/grz_current_test",
@@ -167,21 +209,23 @@ class ReplayManager(object):
                         location = runner.location(
                             "/grz_harness",
                             self.server.port,
-                            close_after=self.target.rl_reset * test_count,
-                            forced_close=self.target.forced_close)
+                            close_after=relaunch * test_count,
+                            timeout=self.server.timeout)
                     # The environment from the initial testcase is used because
                     # a sequence of testcases is expected to be run without
                     # relaunching the Target to match the functionality of
                     # Grizzly. If this is not the case each TestCase should
                     # be run individually.
                     runner.launch(location, env_mod=testcases[0].env_vars)
-                self.target.step()
-                LOG.info("Performing replay (%d/%d)...", self.status.iteration, repeat)
                 # run tests
                 durations = list()
                 served = list()
                 for test_idx in range(test_count):
-                    LOG.debug("running test: %d of %d", test_idx + 1, test_count)
+                    if test_count > 1:
+                        LOG.info("Running test, part %d/%d (%d/%d)...",
+                                 test_idx + 1, test_count, self.status.iteration, repeat)
+                    else:
+                        LOG.info("Running test (%d/%d)...", self.status.iteration, repeat)
                     # update redirects
                     if self._harness is not None:
                         next_idx = (test_idx + 1) % test_count
@@ -202,8 +246,16 @@ class ReplayManager(object):
                         wait_for_callback=self._harness is None)
                     durations.append(run_result.duration)
                     served.append(run_result.served)
-                    if run_result.status != RunResult.COMPLETE:
+                    if run_result.status is not None or not run_result.attempted:
                         break
+                if not run_result.attempted:
+                    if run_result.initial:
+                        err_logs = mkdtemp(prefix="error_", dir=grz_tmp("logs"))
+                        self.target.save_logs(err_logs)
+                        LOG.error("ERROR: Test case was not served. Timeout too short?")
+                        LOG.error("Logs can be found here %r", err_logs)
+                        break
+                    LOG.warning("Test case was not served")
                 # process run results
                 if run_result.status == RunResult.FAILED:
                     log_path = mkdtemp(prefix="logs_", dir=grz_tmp("logs"))
@@ -252,34 +304,32 @@ class ReplayManager(object):
                 elif run_result.status == RunResult.IGNORED:
                     self.status.ignored += 1
                     LOG.info("Result: Ignored (%d)", self.status.ignored)
-                elif run_result.status == RunResult.ERROR:
-                    LOG.error("ERROR: Test case was not served. Timeout too short?")
-                    break
 
-                # check status and exit early if possible
-                if repeat - self.status.iteration + self.status.results < min_results:
-                    if self.status.iteration < repeat:
-                        LOG.debug("skipping remaining attempts")
-                    # failed to reproduce issue
-                    LOG.debug("results (%d) < minimum (%d), after %d attempts",
-                              self.status.results, min_results, self.status.iteration)
-                    break
-                # check if complete (results found)
-                if self.status.results >= min_results:
-                    assert self.status.results == min_results
-                    assert sum(x.count for x in reports.values() if x.expected) >= min_results
-                    LOG.debug("results == expected (%d), after %d attempts",
-                              min_results, self.status.iteration)
-                    break
+                if exit_early:
+                    # failed to meet minimum number of results
+                    if repeat - self.status.iteration + self.status.results < min_results:
+                        if self.status.iteration < repeat:
+                            LOG.debug("skipping remaining attempts")
+                        # failed to reproduce issue
+                        LOG.debug("results (%d) < minimum (%d), after %d attempts",
+                                  self.status.results, min_results, self.status.iteration)
+                        # NOTE: this can be tricky if the harness is used because it can
+                        # skip the shutdown performed in the harness and runner, if this
+                        # is an issue for now use relaunch=1
+                        break
+                    # check if complete (minimum number of results found)
+                    if self.status.results >= min_results:
+                        assert self.status.results == min_results
+                        assert sum(x.count for x in reports.values() if x.expected) >= min_results
+                        LOG.debug("results == expected (%d), after %d attempts",
+                                  min_results, self.status.iteration)
+                        break
 
                 # warn about large browser logs
                 #self.status.log_size = self.target.log_size()
                 #if self.status.log_size > self.TARGET_LOG_SIZE_WARN:
                 #    LOG.warning("Large browser logs: %dMBs", (self.status.log_size / 0x100000))
 
-                # trigger relaunch by closing the browser if needed
-                self.target.check_relaunch()
-
             # process results
             results = list()
             if self._any_crash:
@@ -299,6 +349,11 @@ class ReplayManager(object):
                         report.report.cleanup()
                         continue
                     results.append(report)
+            # this should only be displayed when both conditions are met:
+            # 1) runner does not close target (no delay was given before shutdown)
+            # 2) result has not been successfully reproduced
+            if self._relaunch > 1 and not self.target.closed and not any(x.expected for x in results):
+                LOG.info("Perhaps try with --relaunch=1")
             # active reports have been moved to results
             # clear reports to avoid cleanup of active reports
             reports.clear()
@@ -337,25 +392,26 @@ class ReplayManager(object):
         else:
             signature = None
 
-        LOG.debug("loading the TestCases")
         try:
-            testcases = TestCase.load(args.input, args.prefs is None)
-            if not testcases:
-                raise TestCaseLoadFailure("Failed to load TestCases")
+            testcases = cls.load_testcases(
+                args.input,
+                args.prefs is None,
+                subset=args.test_index)
         except TestCaseLoadFailure as exc:
             LOG.error("Error: %s", str(exc))
-            return 1
+            return Session.EXIT_ERROR
 
         results = None
         target = None
         tmp_prefs = None
         try:
             if args.no_harness and len(testcases) > 1:
-                LOG.error("'--no-harness' cannot be used with multiple testcases")
-                return 1
+                LOG.error(
+                    "'--no-harness' cannot be used with multiple testcases. " \
+                    "Perhaps '--test-index' can help.")
+                return Session.EXIT_ARGS
             repeat = max(args.min_crashes, args.repeat)
             relaunch = min(args.relaunch, repeat)
-            assert not args.no_harness or (args.no_harness and relaunch == 1)
             LOG.info("Repeat: %d, Minimum crashes: %d, Relaunch %d",
                      repeat, args.min_crashes, relaunch)
             LOG.debug("initializing the Target")
@@ -365,7 +421,6 @@ class ReplayManager(object):
                 args.launch_timeout,
                 args.log_limit,
                 args.memory,
-                relaunch,
                 rr=args.rr,
                 valgrind=args.valgrind,
                 xvfb=args.xvfb)
@@ -384,9 +439,6 @@ class ReplayManager(object):
                         LOG.info("Using prefs.js from testcase")
                         target.prefs = pathjoin(tmp_prefs, "prefs.js")
                         break
-            if testcases[0].env_vars.get("GRZ_FORCED_CLOSE") == "0":
-                LOG.debug("setting target.forced_close=False")
-                target.forced_close = False
             LOG.debug("starting sapphire server")
             # launch HTTP server used to serve test cases
             with Sapphire(auto_close=1, timeout=args.timeout) as server:
@@ -396,6 +448,7 @@ class ReplayManager(object):
                     server,
                     target,
                     any_crash=args.any_crash,
+                    relaunch=relaunch,
                     signature=signature,
                     use_harness=not args.no_harness
                 ) as replay:
@@ -417,10 +470,10 @@ class ReplayManager(object):
                     results,
                     testcases if args.include_test else None)
             # TODO: add fuzzmanager reporting
-            return 0 if success else 1
+            return Session.EXIT_SUCCESS if success else Session.EXIT_FAILURE
 
         except KeyboardInterrupt:
-            return 1
+            return Session.EXIT_ABORT
 
         except (TargetLaunchError, TargetLaunchTimeout) as exc:
             LOG.error(str(exc))
@@ -429,10 +482,10 @@ class ReplayManager(object):
                 LOG.error("Logs can be found here %r", path)
                 reporter = FilesystemReporter(path, major_bucket=False)
                 reporter.submit([], exc.report)
-            return 1
+            return Session.EXIT_LAUNCH_FAILURE
 
         finally:
-            LOG.warning("Shutting down...")
+            LOG.info("Shutting down...")
             if results:
                 # cleanup unreported results
                 for result in results:
