@@ -7,7 +7,7 @@ from logging import getLogger
 from os.path import isdir
 from shutil import rmtree
 from tempfile import mkdtemp
-from time import sleep, time
+from time import time
 
 from .common import grz_tmp, Report, Runner, RunResult, Status, TestFile
 
@@ -17,14 +17,14 @@ __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith", "Jesse Schwartzentruber"]
 
 
-log = getLogger(__name__)  # pylint: disable=invalid-name
+LOG = getLogger(__name__)
 
 
 class SessionError(Exception):
     """The base class for exceptions raised by Session"""
 
 
-class LogOutputLimiter(object):
+class LogOutputLimiter:
     __slots__ = ("_delay", "_iterations", "_launches", "_multiplier", "_time", "_verbose")
 
     def __init__(self, delay=300, delta_multiplier=2, verbose=False):
@@ -53,18 +53,24 @@ class LogOutputLimiter(object):
         return ready
 
 
-class Session(object):
+class Session:
     DISPLAY_VERBOSE = 0  # display status every iteration
     DISPLAY_NORMAL = 1  # quickly reduce the amount of output
     EXIT_SUCCESS = 0
-    EXIT_ERROR = 1
-    EXIT_ABORT = 3
-    EXIT_LAUNCH_FAILURE = 7
+    EXIT_ERROR = 1  # unexpected error occurred (invalid input, unhanded exception, etc)
+    EXIT_ARGS = 2  # invalid argument
+    EXIT_ABORT = 3  # run aborted (ctrl+c, etc)
+    EXIT_LAUNCH_FAILURE = 4  # unrelated Target failure (browser startup crash, etc)
+    EXIT_FAILURE = 5  # expected results not reproduced (opposite of EXIT_SUCCESS)
+
     TARGET_LOG_SIZE_WARN = 0x1900000  # display warning when target log files exceed limit (25MB)
 
-    __slots__ = ("adapter", "coverage", "iomanager", "reporter", "server", "status", "target")
+    __slots__ = ("_relaunch", "adapter", "coverage", "iomanager", "reporter", "server",
+                 "status", "target")
 
-    def __init__(self, adapter, iomanager, reporter, server, target, coverage=False):
+    def __init__(self, adapter, iomanager, reporter, server, target,
+                 coverage=False, relaunch=1):
+        self._relaunch = relaunch
         self.adapter = adapter
         self.coverage = coverage
         self.iomanager = iomanager
@@ -84,19 +90,19 @@ class Session(object):
 
     def display_status(self, log_limiter):
         if self.adapter.remaining is not None:
-            log.info(
+            LOG.info(
                 "[I%04d-L%02d-R%02d] %s",
                 self.status.iteration,
                 self.adapter.remaining,
                 self.status.results,
                 self.status.test_name)
         elif log_limiter.ready(self.status.iteration, self.target.monitor.launches):
-            log.info("I%04d-R%02d ", self.status.iteration, self.status.results)
+            LOG.info("I%04d-R%02d ", self.status.iteration, self.status.results)
 
     def generate_testcase(self):
-        log.debug("calling iomanager.create_testcase()")
+        LOG.debug("calling iomanager.create_testcase()")
         test = self.iomanager.create_testcase(self.adapter.NAME)
-        log.debug("calling self.adapter.generate()")
+        LOG.debug("calling self.adapter.generate()")
         self.adapter.generate(test, self.iomanager.server_map)
         self.status.test_name = test.input_fname
         if self.target.prefs is not None:
@@ -110,7 +116,7 @@ class Session(object):
         self.target.save_logs(result_logs)
         report = Report(result_logs, self.target.binary)
         short_sig = report.crash_info.createShortSignature()
-        log.info("Result: %s (%s:%s)", short_sig, report.major[:8], report.minor[:8])
+        LOG.info("Result: %s (%s:%s)", short_sig, report.major[:8], report.minor[:8])
         # order test cases newest to oldest
         self.iomanager.tests.reverse()
         self.reporter.submit(self.iomanager.tests, report)
@@ -120,19 +126,16 @@ class Session(object):
 
     def run(self, ignore, iteration_limit=None, display_mode=DISPLAY_NORMAL):
         log_limiter = LogOutputLimiter(verbose=display_mode == self.DISPLAY_VERBOSE)
-        runner = Runner(self.server, self.target)
-
-        def _dyn_close():  # pragma: no cover
-            if self.target.monitor.is_healthy():
-                # delay to help catch window close/shutdown related crashes
-                sleep(0.1)
-                self.target.close()
-            return b"<h1>Close Browser</h1>"
-        self.iomanager.server_map.set_dynamic_response(
-            "grz_close_browser",
-            _dyn_close,
-            mime_type="text/html")
-
+        # limit relaunch to max iterations if known
+        if iteration_limit is not None:
+            assert iteration_limit > 0
+            relaunch = min(self._relaunch, iteration_limit)
+        else:
+            relaunch = self._relaunch
+        if self.adapter.remaining is not None:
+            assert self.adapter.remaining > 0
+            relaunch = min(relaunch, self.adapter.remaining)
+        runner = Runner(self.server, self.target, relaunch=relaunch)
         while True:
             self.status.report()
             self.status.iteration += 1
@@ -143,20 +146,17 @@ class Session(object):
                 self.adapter.pre_launch()
                 if self.iomanager.harness is None:
                     # harness is not in use, open the test case
-                    location = runner.location(
-                        "/grz_current_test",
-                        self.server.port)
+                    location = runner.location("/grz_current_test", self.server.port)
                 else:
-                    # harness is in use, open it and it will open the test case
+                    # harness is in use, open it and it will open the test case.
+                    # use adapter.TEST_DURATION to allow the harness to attempt to
+                    # close test cases when server timeout is greater.
                     location = runner.location(
                         "/grz_harness",
                         self.server.port,
-                        close_after=self.target.rl_reset,
-                        forced_close=self.target.forced_close,
+                        close_after=relaunch,
                         timeout=self.adapter.TEST_DURATION)
-                log.info("Launching target")
                 runner.launch(location, max_retries=3, retry_delay=0)
-            self.target.step()
 
             # create and populate a test case
             current_test = self.generate_testcase()
@@ -168,49 +168,47 @@ class Session(object):
             current_test.duration = result.duration
             # adapter callbacks
             if result.timeout:
-                log.debug("calling self.adapter.on_timeout()")
+                LOG.debug("calling self.adapter.on_timeout()")
                 self.adapter.on_timeout(current_test, result.served)
             else:
-                log.debug("calling self.adapter.on_served()")
+                LOG.debug("calling self.adapter.on_served()")
                 self.adapter.on_served(current_test, result.served)
             # update test case
-            if result.status != RunResult.ERROR:
-                if not result.served:
-                    # this can happen if the target crashes between serving test cases
-                    log.info("Ignoring test case since nothing was served")
-                    self.iomanager.tests.pop().cleanup()
-                elif self.adapter.IGNORE_UNSERVED:
-                    log.debug("removing unserved files from the test case")
-                    current_test.purge_optional(result.served)
+            if not result.attempted:
+                LOG.debug("Ignoring test case since nothing was served")
+                self.iomanager.tests.pop().cleanup()
+                if not current_test.contains(current_test.landing_page):
+                    LOG.warning("Test case is missing landing page")
+                if result.initial:
+                    # since this is the first iteration since the Target launched
+                    # something is likely wrong with the Target or Adapter
+                    err_logs = mkdtemp(prefix="error_", dir=grz_tmp("logs"))
+                    self.target.save_logs(err_logs)
+                    LOG.error("ERROR: Test case was not served. Timeout too short?")
+                    LOG.error("Logs can be found here %r", err_logs)
+                    raise SessionError("Please check Adapter and Target")
+                LOG.warning("Test case was not served")
+            elif self.adapter.IGNORE_UNSERVED:
+                LOG.debug("removing unserved files from the test case")
+                current_test.purge_optional(result.served)
             # process results
             if result.status == RunResult.FAILED:
-                log.debug("result detected")
+                LOG.debug("result detected")
                 self.report_result()
             elif result.status == RunResult.IGNORED:
                 self.status.ignored += 1
-                log.info("Ignored (%d)", self.status.ignored)
-            elif result.status == RunResult.ERROR:
-                log.error("Test case was not served")
-                if not current_test.contains(current_test.landing_page):
-                    log.warning("Test case is missing landing page")
-                if self.status.iteration < 3:
-                    # since this is the first few iteration something is
-                    # likely wrong with the Target (caching?) or the Adapter
-                    raise SessionError("Please check Adapter and Target")
-
-            # trigger relaunch by closing the browser if needed
-            self.target.check_relaunch()
+                LOG.info("Ignored (%d)", self.status.ignored)
 
             if self.adapter.remaining is not None and self.adapter.remaining < 1:
                 # all test cases have been replayed
-                log.info("Replay Complete")
+                LOG.info("Replay Complete")
                 break
 
             if iteration_limit is not None and self.status.iteration == iteration_limit:
-                log.info("Hit iteration limit")
+                LOG.info("Hit iteration limit")
                 break
 
             # warn about large browser logs
             self.status.log_size = self.target.log_size()
             if self.status.log_size > self.TARGET_LOG_SIZE_WARN:
-                log.warning("Large browser logs: %dMBs", (self.status.log_size / 0x100000))
+                LOG.warning("Large browser logs: %dMBs", (self.status.log_size / 0x100000))
