@@ -1,11 +1,12 @@
 # coding=utf-8
-"""
-Loki fuzzing library
-"""
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from .loki import main
+"""
+Loki fuzzing library
+"""
+from .args import parse_args
+from .loki import Loki
 
 
-main()
+raise SystemExit(Loki.main(parse_args()))
